pub enum NodeRenderingType {
Rendered,
DelegatesRendering,
NotRendered,
}Expand description
Whether or not this node is being rendered or delegates rendering according to the HTML standard.
Variants
Rendered
DelegatesRendering
NotRendered
If neither of the other two cases are true, this is. The node is effectively not taking part in the final layout of the page.