pub struct HitTestResultItem {
pub node: OpaqueNode,
pub point_in_target: Point2D<f32, CSSPixel>,
pub cursor: Cursor,
}Expand description
Describe an item that matched a hit-test query.
Fields
node: OpaqueNodeAn OpaqueNode that contains a pointer to the node hit by
this hit test result.
point_in_target: Point2D<f32, CSSPixel>The Point2D of the original query point relative to the
node fragment rectangle.
cursor: CursorThe Cursor that’s defined on the item that is hit by this
hit test result.