pub struct ImageAnimationState {
pub image: Arc<RasterImage>,
pub active_frame: usize,
pub(crate) frame_start_time: f64,
pub completed_loops: Option<u32>,
}Fields
image: Arc<RasterImage>active_frame: usizeframe_start_time: f64completed_loops: Option<u32>The number of loops that have fully completed in this ImageAnimationState.
If this is greater than or equal to the maximum number of loops in the
RasterImage, then the animation has ended. If it is None, then the image
will loop infinitely.
Implementations
Sourceimpl ImageAnimationState
impl ImageAnimationState
pub fn new(image: Arc<RasterImage>, last_update_time: f64) -> Self
pub fn image_key(&self) -> Option<ImageKey>
pub fn duration_to_next_frame(&self, now: f64) -> Option<Duration>
Sourcepub fn update_frame_for_animation_timeline_value(&mut self, now: f64) -> bool
pub fn update_frame_for_animation_timeline_value(&mut self, now: f64) -> bool
check whether image active frame need to be updated given current time, return true if there are image that need to be updated. false otherwise.
Sourcepub(crate) fn is_finished(&self) -> bool
pub(crate) fn is_finished(&self) -> bool
Whether or not this animation has finished looping and has reached its final frame.
Sourcepub(crate) fn advance_completed_loops(&mut self)
pub(crate) fn advance_completed_loops(&mut self)
If this animation has a finite number of loops, advance the count of completed loops.