pub struct ReflowRequestAccessibility {
pub damage: Vec<(TrustedNodeAddress, AccessibilityDamage)>,
pub rooted_nodes_for_integrity_check: Option<FxHashSet<OpaqueNode>>,
}Expand description
Information needed for a script-initiated reflow that requires the accessibility tree to be updated.
Fields
damage: Vec<(TrustedNodeAddress, AccessibilityDamage)>Damage to the accessibility tree from DOM mutations.
rooted_nodes_for_integrity_check: Option<FxHashSet<OpaqueNode>>Nodes which were removed from the DOM tree since the last reflow, which were rooted in
[AccessibilityData]. Only set if [pref::expensive_accessibility_test_assertions_enabled]
is set.