pub fn with_layout_state<R>(f: impl FnOnce() -> R) -> RExpand description
Set up the thread-local state to reflect that layout code is about to run,
then call the provided function.
This must be used when running code that will interact with the DOM tree
through types like ServoLayoutNode, ServoLayoutElement, and LayoutDom,
which have rules about how they must be used from layout worker threads.